#version 3.7;


//----------------global settings---------------------------------
global_settings{
assumed_gamma 1.0
}


//----------------modify project settings---------------------------------

// file names
#declare output_info_data_file_name = "the_sky_render_info_data"
#declare output_sun_sp_file_name = "the_sky_render_sun_sp_data"


// output settings
#declare projection = 1; // 1: spherical  2: horizon spherical  3: fisheye
#declare half_render = true;


// output brightness
#declare sp_radiance_per_white = pow(10,8); // W/m^2/sr/m/white


// wavelength settings, unit nm
#declare project_min_wavelgth = 380;
#declare project_max_wavelgth = 730;
#declare project_wavelgth_resolution = final_frame - initial_frame + 1; //36
#declare project_wavelgth_step = (project_max_wavelgth - project_min_wavelgth)/(project_wavelgth_resolution - 1); //10


// quality settings
#declare calc_secondary_scatter = false;

#declare sky_light_primary_path_res = 8;
#declare sky_light_secondary_spl_res = 10;
#declare sky_light_secondary_path_res = 3;
#declare sky_light_tertiary_spl_res = 2;
#declare sky_light_tertiary_path_res = 1;
#declare sun_extinct_res = 6;
#declare skylight_extinct_res = 6;


// earth and atmosphere settings
#declare earth_r = 6372; //kilometer
#declare atm_top_altitude = 60; //kilometer

#declare f_rayleigh_extinction_coefficient = // 1/kilometer
function(_altitude, _wavelgth){
0.0007*pow(730/_wavelgth,4.05)
*exp(-_altitude/20)
}

#declare factor_mie_anisotropic1 = 0.92;
#declare f_mie_extinction_coefficient1 = // 1/kilometer
function(_altitude){
0.001
*exp(-_altitude/1.7)
}

#declare factor_mie_anisotropic2 = 0.6;
#declare f_mie_extinction_coefficient2 = // 1/kilometer
function(_altitude){
0.01
*exp(-_altitude/2.4)
}

#declare f_absorb_extinction_coefficient = // 1/kilometer
function(_altitude){
0.015
*exp(-_altitude/2.0)
}

#declare ground_albedo = 0.2;


// sun settings
#declare sun_zenith_angle = 88; // degree
#declare sun_position_marked = true;
#declare sun_angular_diameter = 0.537; // degree

#declare f_sun_sp_radiance = // W/m^2/sr/m
function(_wavelgth){
119.268*pow(10,12)/pow(_wavelgth*0.001,5)/(exp(0.0144/(_wavelgth*pow(10,-9)*5780))-1)
}


// camera settings
#declare camera_altitude = 0.005; // kilometer










//--------------internal calculate-----------------------------------


#declare this_frame_wavelgth = project_min_wavelgth + project_wavelgth_step*(clock-1);
#if(camera_altitude >= atm_top_altitude)
    #error "Does not support camera_altitude >= atm_top_altitude"
#end

#include "math.inc"
#include "sky_render geometric_functions.inc"
#include "sky_render light_calc_functions.inc"


#declare horizon_theta = pi/2 + acos(earth_r/(earth_r+camera_altitude));










//----------------image output-------------------------------------

#declare Csg_Overlap = 0.001;

#declare total_wdth = 1;
#declare total_hght = 1*image_height/image_width;







#declare sun_radiance = f_sun_light_field(0, 0, 0)/sun_steradian; // W/m^2/sr/m

#declare output_max_color = 2000;

#switch(projection)
#case(1)
    #declare p_sky_output =
    pigment{
        function{
        f_sky_radiance_camera_view(pi/2-y*pi/total_hght, -x*2*pi/total_wdth)/sp_radiance_per_white/output_max_color
        }
        color_map{
        [0 rgb 0]
        [1 rgb 1*output_max_color]
        }
    }

    #declare ground_object =
    box{
    <-total_wdth/2, -total_hght/2, 0.0>, <total_wdth/2, total_hght/2-horizon_theta*total_hght/pi, 0.1>
        pigment{rgb sun_radiance/sp_radiance_per_white*cos(sun_theta)*sun_steradian*ground_albedo}
        finish{ambient 1 diffuse 0}
    no_shadow
    }
    
    #declare mask_layer =
    box{
    <0+total_wdth/image_width, -total_hght/2, 0.0>, <total_wdth/2, total_hght/2, 0.05>
        texture{
            pigment{rgb <0,0,0>}
            finish{ambient 0 diffuse 0}
        }
    no_shadow
    }
    
    #declare sun_mark_object =
    union{
        cylinder{<-5/image_width,0,0.2>, <5/image_width,0,0.2>, 1/2/image_width}
        cylinder{<0,-5/image_width,0.2>, <0,5/image_width,0.2>, 1/2/image_width}
        texture{
            pigment{rgb <0,0,0>}
            finish{ambient 0 diffuse 0}
        }
    translate <0, total_hght/2-sun_theta/pi*total_hght, 0>
    }
    
#break
#case(2)
#break
#case(3)
    #declare p_sky_output =
    pigment{
        function{
        f_sky_radiance_camera_view(sqrt(x*x+y*y)*pi/total_wdth, select(y, 2*pi-acos(x/sqrt(x*x+y*y)), acos(x/sqrt(x*x+y*y))))/sp_radiance_per_white/output_max_color
        }
        color_map{
        [0 rgb 0]
        [1 rgb 1*output_max_color]}
    }
    
    #declare ground_object =
    difference{
        box{
        <-total_wdth/2, -total_hght/2, 0.0>, <total_wdth/2, total_hght/2, 0.1>
        }
        cylinder{
        <0,0,0-Csg_Overlap>, <0,0,0.1+Csg_Overlap>, horizon_theta*total_wdth/pi
        }
        texture{
            pigment{rgb sun_radiance/sp_radiance_per_white*cos(sun_theta)*sun_steradian*ground_albedo}
            finish{ambient 1 diffuse 0}
        }
    no_shadow
    }

    #declare mask_layer =
    box{
    <-total_wdth/2, -total_hght/2, 0.0>, <total_wdth/2, 0-total_hght/image_height, 0.05>
        texture{
            pigment{rgb <0,0,0>}
            finish{ambient 0 diffuse 0}
        }
    no_shadow
    }

    #declare sun_mark_object =
    union{
        cylinder{<-5/image_width,0,0.2>, <5/image_width,0,0.2>, 1/2/image_width}
        cylinder{<0,-5/image_width,0.2>, <0,5/image_width,0.2>, 1/2/image_width}
        texture{
            pigment{color rgb <0,0,0>}
            finish{ambient 0 diffuse 0}
        }
    translate <sun_theta/pi*total_wdth, 0, 0>
    }
#end


object{
box{<-total_wdth/2, -total_hght/2, -0.1>, <total_wdth/2, total_hght/2, 0.0>}
    texture{
        pigment{p_sky_output}
        finish{ambient 1 diffuse 0}
    }
}

object{
ground_object
}

#if(half_render = true)
    object{
    mask_layer
    }
#end

#if(sun_position_marked)
    object{
    sun_mark_object
    }
#end



camera{
orthographic
location <0, 0, 1>
look_at <0, 0, 0>
right <-1*total_wdth, 0, 0>
up <0, total_hght, 0>
translate <0,0,0>
}






//--------------text information output-----------------------------------

#include "CIE.inc"

#declare sun_outspace_powr = 0; //in this wavelength range
#for(k, 0, project_wavelgth_resolution-1)
    #declare sun_outspace_powr = sun_outspace_powr + f_sun_sp_radiance(project_min_wavelgth + k*project_wavelgth_step)*project_wavelgth_step*pow(10,-9)*sun_steradian; // W/m^2
#end

#declare this_sun_powr = f_sun_light_field(0, 0, 0)*project_wavelgth_step*pow(10,-9); // W/m^2
    


#if(frame_number = initial_frame)

    #fopen info_data_file concat(output_info_data_file_name, ".txt") write
    #write (info_data_file, "Wavelength(nm) =\n")
    #for(k, 0, project_wavelgth_resolution-1)
        #write (info_data_file, str(project_min_wavelgth+project_wavelgth_step*k,0,0), ",\n")
    #end
    #write (info_data_file, "\n\n")
    #write (info_data_file, "Spectral radiance per white = ", str(sp_radiance_per_white,0,0), ",\n\n")
    #write (info_data_file, "Solar zenith angle = ", str(sun_zenith_angle,0,0), " degrees,\n\n")
    #fclose info_data_file

    #fopen solar_sp_data_file concat(output_sun_sp_file_name, ".txt") write
    #write (solar_sp_data_file, str(this_sun_powr,0,3), ",\n")
    #fclose solar_sp_data_file

#elseif(frame_number = final_frame)

    #fopen solar_sp_data_file concat(output_sun_sp_file_name, ".txt") append
    #write (solar_sp_data_file, str(this_sun_powr,0,3), ",\n")
    #fclose solar_sp_data_file

    #declare sun_irradiance = array[project_wavelgth_resolution]
    #fopen solar_sp_data_file concat(output_sun_sp_file_name, ".txt") read
    #for(k, 0, project_wavelgth_resolution-1)
        #read (solar_sp_data_file, sun_irradiance_temp)
        #declare sun_irradiance[k] = sun_irradiance_temp;
    #end
    #declare sun_powr = 0;
    #declare sun_illuminace = 0;
    #declare sun_XYZ = <0,0,0>;
    #for(k, 0, project_wavelgth_resolution-1)
        #declare sun_powr = sun_powr + sun_irradiance[k];
        #declare sun_illuminace = sun_illuminace + sun_irradiance[k]*683.002*CMF_xyz(project_min_wavelgth + k*project_wavelgth_step).y;
        #declare sun_XYZ = sun_XYZ + sun_irradiance[k]*683.002*CMF_xyz(project_min_wavelgth + k*project_wavelgth_step)/sun_steradian;
    #end
    #declare sun_color =
    <csMatRGB[0][0]*sun_XYZ.x + csMatRGB[0][1]*sun_XYZ.y + csMatRGB[0][2]*sun_XYZ.z,
    csMatRGB[1][0]*sun_XYZ.x + csMatRGB[1][1]*sun_XYZ.y + csMatRGB[1][2]*sun_XYZ.z,
    csMatRGB[2][0]*sun_XYZ.x + csMatRGB[2][1]*sun_XYZ.y + csMatRGB[2][2]*sun_XYZ.z>*pow(10,9)/(project_max_wavelgth-project_min_wavelgth)/sp_radiance_per_white/683.002;

    #fopen info_data_file concat(output_info_data_file_name, ".txt") append
    #write (info_data_file, "Solar outspace power in the wavelength range = ", str(sun_outspace_powr,0,2), " W/m^2,\n\n")
    #write (info_data_file, "Solar power to camera in the wavelength range = ", str(sun_powr,0,2), " W/m^2,\n\n")
    #write (info_data_file, "Solar illuminance to camera = ", str(sun_illuminace,0,2), " lux,\n\n")
    #write (info_data_file, "Sun disk color after compose = <", str(sun_color.x,0,2), ", ", str(sun_color.y,0,2), ", ", str(sun_color.z,0,2), ">, (D65) Correctness not sure\n\n")
    #write (info_data_file, "Sun parallel light color after compose = <", str(sun_color.x*sun_steradian,0,2), ", ", str(sun_color.y*sun_steradian,0,2), ", ", str(sun_color.z*sun_steradian,0,2), ">, (D65) Correctness not sure\n\n")
    #fclose info_data_file

#else

    #fopen solar_sp_data_file concat(output_sun_sp_file_name, ".txt") append
    #write (solar_sp_data_file, str(this_sun_powr,0,3), ",\n")
    #fclose solar_sp_data_file


#end



